﻿using System;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Documents.Model;
using System.Windows.Media;
using Telerik.Windows.Documents.UI.Extensibility;
using System.Linq;

namespace CustomTableBordersDialog
{
    /// <summary>
    /// Interaction logic for MyCellBordersProperties.xaml
    /// </summary>
    public partial class MyCellBordersProperties : UserControl
    {
        #region Fields

        private TablePropertiesEditor tablePropertiesEditor;

        #endregion

        #region Properties

        public Telerik.Windows.Documents.Model.Border TopBorder { get; set; }
        public Telerik.Windows.Documents.Model.Border BottomBorder { get; set; }
        public Telerik.Windows.Documents.Model.Border LeftBorder { get; set; }
        public Telerik.Windows.Documents.Model.Border RightBorder { get; set; }

        #endregion


        public MyCellBordersProperties()
        {
            InitializeComponent();

            this.TopBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);
            this.BottomBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);
            this.LeftBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);
            this.RightBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);


            this.richTextPreview.Document.Measure(RadDocument.MAX_DOCUMENT_SIZE);
            this.richTextPreview.UpdateEditorLayout();
            this.richTextPreview.Document.CaretPosition.Reset();
        }
        #region Methods

        public void FillUI(TablePropertiesEditor tablePropertiesEditor)
        {
            this.tablePropertiesEditor = tablePropertiesEditor;
            this.btnBoxBorder.IsChecked = true;

            this.GetValuesFromTable();
        }

        private void GetValuesFromTable()
        {
            TableCell firstCell = this.tablePropertiesEditor.GetTargetCells().FirstOrDefault();
            if (firstCell != null)
            {
                Telerik.Windows.Documents.Model.TableCellBorders borders = new Telerik.Windows.Documents.Model.TableCellBorders(firstCell.Borders);

                this.TopBorder = (borders.Top.Style == BorderStyle.Inherit) ? new Telerik.Windows.Documents.Model.Border(BorderStyle.Single) : borders.Top;
                this.BottomBorder = (borders.Bottom.Style == BorderStyle.Inherit) ? new Telerik.Windows.Documents.Model.Border(BorderStyle.Single) : borders.Bottom;
                this.LeftBorder = (borders.Left.Style == BorderStyle.Inherit) ? new Telerik.Windows.Documents.Model.Border(BorderStyle.Single) : borders.Left;
                this.RightBorder = (borders.Right.Style == BorderStyle.Inherit) ? new Telerik.Windows.Documents.Model.Border(BorderStyle.Single) : borders.Right;

                this.borderSelector.SelectedBorder = borders.Left;

                this.UpdatePreview();
            }
        }
        public void SaveChanges()
        {
            TableCellBorders borders = new TableCellBorders(this.LeftBorder, this.TopBorder, this.RightBorder, this.BottomBorder);

            if (this.tablePropertiesEditor.GetCurrentTable().CellSpacing != 0)
            {
                this.tablePropertiesEditor.ClearSelectedCellBorders();
            }

            this.tablePropertiesEditor.SetTargetCellsBorders(borders);

            this.tablePropertiesEditor.ExecuteCommands();
        }

        private void UpdatePreview()
        {
            Telerik.Windows.Documents.Model.TableBorders borders =
                new Telerik.Windows.Documents.Model.TableBorders(this.LeftBorder, this.TopBorder, this.RightBorder, this.BottomBorder);

            this.richTextPreview.ChangeTableBorders(borders);

            this.UpdateButtons();
        }

        private void UpdateButtons()
        {
            this.btnTop.IsChecked = (this.TopBorder.Style != BorderStyle.None);
            this.btnBottom.IsChecked = (this.BottomBorder.Style != BorderStyle.None);
            this.btnLeft.IsChecked = (this.LeftBorder.Style != BorderStyle.None);
            this.btnRight.IsChecked = (this.RightBorder.Style != BorderStyle.None);
        }

        private Telerik.Windows.Documents.Model.Border GetSelectedBorder()
        {
            return this.borderSelector.SelectedBorder;
        }

        private void SetBoxBorder()
        {
            Telerik.Windows.Documents.Model.Border boxBorder = this.borderSelector.SelectedBorder;
            this.TopBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.BottomBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.LeftBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.RightBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);

            this.UpdatePreview();
        }

        public void SetAllValuesToDefault()
        {
            this.btnBoxBorder.IsChecked = true;
            this.borderSelector.SetAllValuesToDefault();
        }

        internal void ClearReferences()
        {
            this.tablePropertiesEditor = null;
        }
        #endregion

        #region Event Handlers

        private void LayoutRoot_Loaded(object sender, RoutedEventArgs e)
        {
            this.UpdatePreview();
        }

        private void btnTop_Click(object sender, RoutedEventArgs e)
        {
            if (this.TopBorder == this.borderSelector.SelectedBorder)
            {
                this.TopBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.TopBorder = this.GetSelectedBorder();
            }

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }

        private void btnBottom_Click(object sender, RoutedEventArgs e)
        {
            if (this.BottomBorder == this.borderSelector.SelectedBorder)
            {
                this.BottomBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.BottomBorder = this.GetSelectedBorder();
            }

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }

        private void btnLeft_Click(object sender, RoutedEventArgs e)
        {
            if (this.LeftBorder == this.borderSelector.SelectedBorder)
            {
                this.LeftBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.LeftBorder = this.GetSelectedBorder();
            }

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }

        private void btnRight_Click(object sender, RoutedEventArgs e)
        {
            if (this.RightBorder == this.borderSelector.SelectedBorder)
            {
                this.RightBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.RightBorder = this.GetSelectedBorder();
            }

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }


        private void borderSelector_SelectedBorderChanged(object sender, EventArgs e)
        {
            if (this.btnBoxBorder.IsChecked.HasValue && this.btnBoxBorder.IsChecked.Value)
            {
                this.SetBoxBorder();
            }
            else if (this.btnNoneBorder.IsChecked.HasValue && this.btnNoneBorder.IsChecked.Value)
            {
                this.btnBoxBorder.IsChecked = true;
            }
        }

        private void btnNoneBorder_Checked(object sender, RoutedEventArgs e)
        {
            this.TopBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            this.BottomBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            this.LeftBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            this.RightBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);

            this.UpdatePreview();
        }

        private void btnBoxBorder_Checked(object sender, RoutedEventArgs e)
        {
            this.SetBoxBorder();
        }

        #endregion
    }
}