﻿using System;
using System.Windows;
using System.Windows.Input;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.RichTextBoxUI.Dialogs;
using Telerik.Windows.Documents.UI.Extensibility;

namespace CustomTableBordersDialog
{
    /// <summary>
    /// Interaction logic for MyTableBordersDialog.xaml
    /// </summary>
    [CustomTableBordersDialog]
    public partial class MyTableBordersDialog : RadRichTextBoxWindow, ITableBordersDialog
    {
        #region Fields

        private TablePropertiesEditor tablePropertiesEditor;

        #endregion

        #region Constructors

        public MyTableBordersDialog()
        {
            InitializeComponent();
        }
        #endregion

        #region Methods

        public void ShowDialog(TablePropertiesEditor tablePropertiesEditor, RadRichTextBox owner)
        {
            this.ShowDialogInternal(tablePropertiesEditor, owner);
        }

        private void ShowDialogInternal(TablePropertiesEditor tablePropertiesEditor, RadRichTextBox owner)
        {
            this.tablePropertiesEditor = tablePropertiesEditor;
            this.SetOwner(owner);

            this.cellBordersControl.FillUI(this.tablePropertiesEditor);
            this.tableBordersControl.FillUI(this.tablePropertiesEditor);
            this.tabControl.SelectedItem = this.tableBordersTab;
            this.ShowDialog();
        }

        private void SaveChanges()
        {
            if (this.tabControl.SelectedItem == this.cellBordersTab)
            {
                this.cellBordersControl.SaveChanges();
            }

            if (this.tabControl.SelectedItem == this.tableBordersTab)
            {
                this.tableBordersControl.SaveChanges();
            }
        }

        private void ResetAllValues()
        {
            this.cellBordersControl.SetAllValuesToDefault();
            this.tableBordersControl.SetAllValuesToDefault();
        }

        protected override void OnClosed(WindowClosedEventArgs args)
        {
            base.OnClosed(args);

            this.tablePropertiesEditor = null;
            this.Owner = null;
            this.ClearReferences();
        }

        private void ClearReferences()
        {
            this.cellBordersControl.ClearReferences();
            this.tableBordersControl.ClearReferences();
        }

        #endregion

        #region Event Handlers

        private void RadWindow_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Escape)
            {
                this.Close();
            }
        }

        private void buttonOK_Click(object sender, RoutedEventArgs e)
        {
            this.SaveChanges();
            this.Close();
        }


        private void buttonCancel_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void buttonResetAll_Click(object sender, RoutedEventArgs e)
        {
            this.ResetAllValues();
        }

        #endregion

    }
}
