﻿using System;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Documents.Model;
using System.Windows.Media;
using Telerik.Windows.Documents.UI.Extensibility;
using System.Linq;

namespace CustomTableBordersDialog
{
    /// <summary>
    /// Interaction logic for MyTableBordersProperties.xaml
    /// </summary>
    public partial class MyTableBordersProperties : UserControl
    {
        #region Fields

        private TablePropertiesEditor tablePropertiesEditor;

        #endregion

        #region Properties

        public Telerik.Windows.Documents.Model.Border TopBorder { get; set; }
        public Telerik.Windows.Documents.Model.Border BottomBorder { get; set; }
        public Telerik.Windows.Documents.Model.Border LeftBorder { get; set; }
        public Telerik.Windows.Documents.Model.Border RightBorder { get; set; }
        public Telerik.Windows.Documents.Model.Border InsideHorizontal { get; set; }
        public Telerik.Windows.Documents.Model.Border InsideVertical { get; set; }

        #endregion

        public MyTableBordersProperties()
        {
            InitializeComponent();

            this.TopBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);
            this.BottomBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);
            this.LeftBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);
            this.RightBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);
            this.InsideHorizontal = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);
            this.InsideVertical = new Telerik.Windows.Documents.Model.Border(BorderStyle.Single);

            this.richTextPreview.Document.Measure(RadDocument.MAX_DOCUMENT_SIZE);
            this.richTextPreview.UpdateEditorLayout();
            this.richTextPreview.Document.CaretPosition.Reset();
        }

        #region Methods

        public void FillUI(TablePropertiesEditor tablePropertiesEditor)
        {
            this.tablePropertiesEditor = tablePropertiesEditor;
            this.btnAllBorder.IsChecked = true;

            this.GetValuesFromTable();
        }

        private void GetValuesFromTable()
        {
            TableCell firstCell = this.tablePropertiesEditor.GetTargetCells().FirstOrDefault();
            if (firstCell != null)
            {
                Telerik.Windows.Documents.Model.TableBorders borders = new Telerik.Windows.Documents.Model.TableBorders(firstCell.Row.Table.Borders);

                this.TopBorder = borders.Top;
                this.BottomBorder = borders.Bottom;
                this.LeftBorder = borders.Left;
                this.RightBorder = borders.Right;
                this.InsideHorizontal = borders.InsideHorizontal;
                this.InsideVertical = borders.InsideVertical;

                this.borderSelector.SelectedBorder = borders.Left;

                this.UpdatePreview();
            }
        }

        public void SaveChanges()
        {
            Telerik.Windows.Documents.Model.TableBorders borders = this.GetSelectedTableBorders();

            //clear all cell borders first so the changes of table borders are visible
            this.tablePropertiesEditor.ClearAllTableCellBorders();
            this.tablePropertiesEditor.SetTableBorders(borders);
            this.tablePropertiesEditor.ExecuteCommands();
        }

        private void UpdatePreview()
        {
            Telerik.Windows.Documents.Model.TableBorders borders = this.GetSelectedTableBorders();

            this.richTextPreview.ChangeTableBorders(borders);

            this.UpdateButtons();
        }

        private Telerik.Windows.Documents.Model.TableBorders GetSelectedTableBorders()
        {
            Telerik.Windows.Documents.Model.TableBorders borders = new Telerik.Windows.Documents.Model.TableBorders(this.LeftBorder, this.TopBorder, this.RightBorder, this.BottomBorder,
                this.InsideHorizontal, this.InsideVertical);

            return borders;
        }

        private void UpdateButtons()
        {
            this.btnTop.IsChecked = (this.TopBorder.Style != BorderStyle.None);
            this.btnBottom.IsChecked = (this.BottomBorder.Style != BorderStyle.None);
            this.btnLeft.IsChecked = (this.LeftBorder.Style != BorderStyle.None);
            this.btnRight.IsChecked = (this.RightBorder.Style != BorderStyle.None);
            this.btnInsideHorizontal.IsChecked = (this.InsideHorizontal.Style != BorderStyle.None);
            this.btnInsideVertical.IsChecked = (this.InsideVertical.Style != BorderStyle.None);
        }

        private Telerik.Windows.Documents.Model.Border GetSelectedBorder()
        {
            return this.borderSelector.SelectedBorder;
        }


        private void ToggleInsideHorizontalBorder()
        {
            if (this.InsideHorizontal == this.borderSelector.SelectedBorder)
            {
                this.InsideHorizontal = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.InsideHorizontal = this.GetSelectedBorder();
            }
        }

        private void ToggleInsideVerticalBorder()
        {
            if (this.InsideVertical == this.borderSelector.SelectedBorder)
            {
                this.InsideVertical = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.InsideVertical = this.GetSelectedBorder();
            }
        }

        private void SetNoneBorders()
        {
            this.TopBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            this.BottomBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            this.LeftBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            this.RightBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            this.InsideHorizontal = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            this.InsideVertical = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);

            this.UpdatePreview();
        }

        private void SetBoxBorders()
        {
            Telerik.Windows.Documents.Model.Border boxBorder = this.borderSelector.SelectedBorder;

            this.TopBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.BottomBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.LeftBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.RightBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.InsideHorizontal = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            this.InsideVertical = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);

            this.UpdatePreview();
        }

        private void SetGridBorders()
        {
            Telerik.Windows.Documents.Model.Border boxBorder = this.borderSelector.SelectedBorder;

            this.TopBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.BottomBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.LeftBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.RightBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.InsideHorizontal = new Telerik.Windows.Documents.Model.Border(1, BorderStyle.Single, Colors.Black);
            this.InsideVertical = new Telerik.Windows.Documents.Model.Border(1, BorderStyle.Single, Colors.Black);

            this.UpdatePreview();
        }

        private void SetAllBorders()
        {
            Telerik.Windows.Documents.Model.Border boxBorder = this.borderSelector.SelectedBorder;

            this.TopBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.BottomBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.LeftBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.RightBorder = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.InsideHorizontal = new Telerik.Windows.Documents.Model.Border(boxBorder);
            this.InsideVertical = new Telerik.Windows.Documents.Model.Border(boxBorder);

            this.UpdatePreview();
        }

        public void SetAllValuesToDefault()
        {
            if (btnAllBorder.IsChecked == true)
            {
                this.btnNoneBorder.IsChecked = true;
                this.btnAllBorder.IsChecked = true;
            }
            else
            {
                this.btnAllBorder.IsChecked = true;
            }
            this.borderSelector.SetAllValuesToDefault();
        }

        internal void ClearReferences()
        {
            this.tablePropertiesEditor = null;
        }
        #endregion

        #region Event Handlers

        private void LayoutRoot_Loaded(object sender, RoutedEventArgs e)
        {
            this.UpdatePreview();
        }

        private void btnTop_Click(object sender, RoutedEventArgs e)
        {
            if (this.TopBorder == this.borderSelector.SelectedBorder)
            {
                this.TopBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.TopBorder = this.GetSelectedBorder();
            }

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }

        private void btnBottom_Click(object sender, RoutedEventArgs e)
        {
            if (this.BottomBorder == this.borderSelector.SelectedBorder)
            {
                this.BottomBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.BottomBorder = this.GetSelectedBorder();
            }

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }

        private void btnLeft_Click(object sender, RoutedEventArgs e)
        {
            if (this.LeftBorder == this.borderSelector.SelectedBorder)
            {
                this.LeftBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.LeftBorder = this.GetSelectedBorder();
            }

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }

        private void btnRight_Click(object sender, RoutedEventArgs e)
        {
            if (this.RightBorder == this.borderSelector.SelectedBorder)
            {
                this.RightBorder = new Telerik.Windows.Documents.Model.Border(BorderStyle.None);
            }
            else
            {
                this.RightBorder = this.GetSelectedBorder();
            }

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }

        private void btnInsideHorizontal_Click(object sender, RoutedEventArgs e)
        {
            this.ToggleInsideHorizontalBorder();

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }

        private void btnInsideVertical_Click(object sender, RoutedEventArgs e)
        {
            this.ToggleInsideVerticalBorder();

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }

        private void btnInsideCenter_Click(object sender, RoutedEventArgs e)
        {
            this.ToggleInsideVerticalBorder();
            this.ToggleInsideHorizontalBorder();

            this.btnCustomBorder.IsChecked = true;
            this.UpdatePreview();
        }



        private void btnNoneBorder_Checked(object sender, RoutedEventArgs e)
        {
            this.SetNoneBorders();
        }

        private void btnBoxBorder_Checked(object sender, RoutedEventArgs e)
        {
            this.SetBoxBorders();
        }

        private void btnAllBorder_Checked(object sender, RoutedEventArgs e)
        {
            this.SetAllBorders();
        }

        private void btnGridBorder_Checked(object sender, RoutedEventArgs e)
        {
            this.SetGridBorders();
        }

        private void borderSelector_SelectedBorderChanged(object sender, EventArgs e)
        {
            if (this.btnNoneBorder.IsChecked.HasValue && this.btnNoneBorder.IsChecked.Value)
            {
                this.btnBoxBorder.IsChecked = true;
            }
            else if (this.btnBoxBorder.IsChecked.HasValue && this.btnBoxBorder.IsChecked.Value)
            {
                this.SetBoxBorders();
            }
            else if (this.btnGridBorder.IsChecked.HasValue && this.btnGridBorder.IsChecked.Value)
            {
                this.SetGridBorders();
            }
            else if (this.btnAllBorder.IsChecked.HasValue && this.btnAllBorder.IsChecked.Value)
            {
                this.SetAllBorders();
            }
        }


        #endregion
    }
}
